use crate::prelude::*;

use std::os::raw::c_void;
use std::ptr;

#[repr(C)]
#[derive(Copy, Clone)]
pub struct VkAccelerationStructureGeometryTrianglesDataKHR {
    pub sType: VkStructureType,
    pub pNext: *const c_void,
    pub vertexFormat: VkFormat,
    pub vertexData: VkDeviceOrHostAddressConstKHR,
    pub vertexStride: VkDeviceSize,
    pub maxVertex: u32,
    pub indexType: VkIndexType,
    pub indexData: VkDeviceOrHostAddressConstKHR,
    pub transformData: VkDeviceOrHostAddressConstKHR,
}

impl VkAccelerationStructureGeometryTrianglesDataKHR {
    pub fn new(
        vertex_format: VkFormat,
        vertex_data: VkDeviceOrHostAddressConstKHR,
        vertex_stride: VkDeviceSize,
        max_vertex: u32,
        index_type: VkIndexType,
        index_data: VkDeviceOrHostAddressConstKHR,
        transform_data: VkDeviceOrHostAddressConstKHR,
    ) -> Self {
        VkAccelerationStructureGeometryTrianglesDataKHR {
            sType: VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_TRIANGLES_DATA_KHR,
            pNext: ptr::null(),
            vertexFormat: vertex_format,
            vertexData: vertex_data,
            vertexStride: vertex_stride,
            maxVertex: max_vertex,
            indexType: index_type,
            indexData: index_data,
            transformData: transform_data,
        }
    }
}
